@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 # pycoast, Writing of coastlines, borders and rivers to images in Python
 #
-# Copyright (C) 2011-2018 PyCoast Developers
+# Copyright (C) 2011-2020 PyCoast Developers
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@ import numpy as np
 from PIL import Image, ImageFont
 import time
 
+
 def tmp(f):
     f.tmp = True
     return f
@@ -357,6 +358,45 @@ class TestPIL(TestPycoast):
         self.assertTrue(fft_metric(grid_data, res),
                         'Writing of nh polygons failed')
 
+    def test_add_points_pil(self):
+        from pycoast import ContourWriterPIL
+        from pyresample.geometry import AreaDefinition
+
+        font_file = os.path.join(os.path.dirname(__file__), 'test_data',
+                                 'DejaVuSerif.ttf')
+        grid_img = Image.open(os.path.join(os.path.dirname(__file__),
+                                           'nh_points_pil.png'))
+        grid_data = np.array(grid_img)
+
+        img = Image.new('RGB', (1024, 1024), (255, 255, 255))
+
+        proj4_string = '+proj=laea +lat_0=90 +lon_0=0 +a=6371228.0 +units=m'
+        area_extent = (-5326849.0625, -5326849.0625,
+                       5326849.0625, 5326849.0625)
+
+        area_def = AreaDefinition('nh', 'nh', 'nh', proj4_string,
+                                  1024, 1024, area_extent)
+
+        cw = ContourWriterPIL(gshhs_root_dir)
+        cw.add_coastlines(img, area_def, outline='black', resolution='l',
+                          level=4)
+        cw.add_borders(img, area_def, outline='black', level=1,
+                       resolution='c')
+
+        points_list = [((13.4050, 52.5200), 'Berlin')]
+        cw.add_points(img, area_def, points_list=points_list, font_file=font_file,
+                      symbol='asterisk', ptsize=6, outline='red',
+                      box_outline='black')
+
+        points_list = [((12.4964, 41.9028), 'Rome')]
+        cw.add_points(img, area_def, points_list=points_list, font_file=font_file,
+                      symbol='square', ptsize=6, outline='blue', fill='yellow',
+                      box_outline='black')
+
+        res = np.array(img)
+        self.assertTrue(fft_metric(grid_data, res),
+                        'Writing of nh points failed')
+
     def test_add_shapefile_shapes(self):
         from pycoast import ContourWriterPIL
         grid_img = Image.open(os.path.join(os.path.dirname(__file__),
@@ -412,6 +452,34 @@ class TestPIL(TestPycoast):
         self.assertTrue(fft_metric(grid_data, res),
                         'Writing of nh grid failed')
 
+    def test_config_file_points_and_borders_pil(self):
+        from pycoast import ContourWriterPIL
+        from pyresample.geometry import AreaDefinition
+
+        config_file = os.path.join(os.path.dirname(__file__),
+                                   'nh_points_pil.ini')
+
+        grid_img = Image.open(os.path.join(os.path.dirname(__file__),
+                                           'nh_points_cfg_pil.png'))
+        grid_data = np.array(grid_img)
+
+        img = Image.new('RGB', (1024, 1024), (255, 255, 255))
+
+        proj4_string = '+proj=laea +lat_0=90 +lon_0=0 +a=6371228.0 +units=m'
+        area_extent = (-5326849.0625, -5326849.0625,
+                       5326849.0625, 5326849.0625)
+
+        area_def = AreaDefinition('nh', 'nh', 'nh', proj4_string,
+                                  1024, 1024, area_extent)
+
+        cw = ContourWriterPIL(gshhs_root_dir)
+
+        cw.add_overlay_from_config(config_file, area_def, img)
+
+        res = np.array(img)
+        self.assertTrue(fft_metric(grid_data, res),
+                        'Writing of nh points failed')
+
 
 class TestPILAGG(TestPycoast):
 
@@ -517,9 +585,9 @@ class TestPILAGG(TestPycoast):
         cw = ContourWriterAGG(gshhs_root_dir)
 
         cw.add_coastlines(img, area_def, resolution='l', level=4)
-        font = aggdraw.Font('blue', os.path.join(os.path.dirname(__file__),
-                                                 'test_data',
-                                                 'DejaVuSerif.ttf'), size=16,
+        font = aggdraw.Font('blue',
+                            os.path.join(os.path.dirname(__file__), 'test_data', 'DejaVuSerif.ttf'),
+                            size=16,
                             opacity=200)
         cw.add_grid(img, area_def, (10.0, 10.0), (2.0, 2.0), font=font,
                     outline='blue', outline_opacity=255, width=1.0,
@@ -644,6 +712,43 @@ class TestPILAGG(TestPycoast):
         self.assertTrue(fft_metric(grid_data, res),
                         'Writing of nh polygons failed')
 
+    def test_add_points_agg(self):
+        from pycoast import ContourWriterAGG
+        from pyresample.geometry import AreaDefinition
+
+        font_file = os.path.join(os.path.dirname(__file__), 'test_data',
+                                 'DejaVuSerif.ttf')
+
+        grid_img = Image.open(os.path.join(os.path.dirname(__file__),
+                                           'nh_points_agg.png'))
+        grid_data = np.array(grid_img)
+
+        img = Image.new('RGB', (1024, 1024), (255, 255, 255))
+        proj4_string = '+proj=laea +lat_0=90 +lon_0=0 +a=6371228.0 +units=m'
+        area_extent = (-5326849.0625, -5326849.0625,
+                       5326849.0625, 5326849.0625)
+
+        area_def = AreaDefinition('nh', 'nh', 'nh', proj4_string,
+                                  1024, 1024, area_extent)
+
+        cw = ContourWriterAGG(gshhs_root_dir)
+        cw.add_coastlines(img, area_def, outline='black', resolution='l',
+                          level=4)
+        cw.add_borders(img, area_def, outline='black', width=3, level=1,
+                       resolution='c')
+
+        points_list = [((2.3522, 48.8566), 'Paris'),
+                       ((0.1278, 51.5074), 'London')]
+        cw.add_points(img, area_def, points_list=points_list, font_file=font_file,
+                      symbol='circle', ptsize=16,
+                      outline='black', width=3,
+                      fill='red', fill_opacity=128,
+                      box_outline='blue', box_linewidth=0.5,
+                      box_fill='yellow', box_opacity=200)
+
+        res = np.array(img)
+        self.assertTrue(fft_metric(grid_data, res), 'Writing of nh points failed')
+
     def test_add_shapefile_shapes_agg(self):
         from pycoast import ContourWriterAGG
         grid_img = Image.open(os.path.join(os.path.dirname(__file__),
@@ -677,31 +782,59 @@ class TestPILAGG(TestPycoast):
         self.assertTrue(
             fft_metric(grid_data, res), 'Writing of Brazil shapefiles failed')
 
-    @unittest.skip("All kwargs are not supported, so can't create equal results")
+#    @unittest.skip("All kwargs are not supported, so can't create equal results")
     def test_config_file_coasts_and_grid(self):
         from pycoast import ContourWriterAGG
         from pyresample.geometry import AreaDefinition
         overlay_config = os.path.join(os.path.dirname(__file__),
                                       "coasts_and_grid_agg.ini")
         grid_img = Image.open(os.path.join(os.path.dirname(__file__),
-                                           'grid_nh_agg.png'))
+                                           'grid_nh_cfg_agg.png'))
         grid_data = np.array(grid_img)
         proj_dict = {'proj': 'laea', 'lat_0': 90.0, 'lon_0': 0.0,
                      'a': 6371228.0, 'units': 'm'}
         area_extent = (-5326849.0625, -5326849.0625,
                        5326849.0625, 5326849.0625)
-        area_def = AreaDefinition('nh', 'nh', 'nh', proj_dict, 425, 425,
+        area_def = AreaDefinition('nh', 'nh', 'nh', proj_dict, 850, 850,
                                   area_extent)
 
         cw = ContourWriterAGG(gshhs_root_dir)
         overlay = cw.add_overlay_from_config(overlay_config, area_def)
-        img = Image.new('RGB', (425, 425))
+        img = Image.new('RGB', (850, 850), (255, 255, 255))
         img.paste(overlay, mask=overlay)
 
         res = np.array(img)
         self.assertTrue(fft_metric(grid_data, res),
                         'Writing of nh grid failed')
 
+    def test_config_file_points_and_borders_agg(self):
+        from pycoast import ContourWriterAGG
+        from pyresample.geometry import AreaDefinition
+
+        config_file = os.path.join(os.path.dirname(__file__),
+                                   'nh_points_agg.ini')
+
+        grid_img = Image.open(os.path.join(os.path.dirname(__file__),
+                                           'nh_points_agg.png'))
+        grid_data = np.array(grid_img)
+
+        img = Image.new('RGB', (1024, 1024), (255, 255, 255))
+
+        proj4_string = '+proj=laea +lat_0=90 +lon_0=0 +a=6371228.0 +units=m'
+        area_extent = (-5326849.0625, -5326849.0625,
+                       5326849.0625, 5326849.0625)
+
+        area_def = AreaDefinition('nh', 'nh', 'nh', proj4_string,
+                                  1024, 1024, area_extent)
+
+        cw = ContourWriterAGG(gshhs_root_dir)
+
+        cw.add_overlay_from_config(config_file, area_def, img)
+
+        res = np.array(img)
+        self.assertTrue(fft_metric(grid_data, res),
+                        'Add points with agg module from a config file failed')
+
     def test_coastlines_convert_to_rgba_agg(self):
         from pycoast import ContourWriterAGG
         proj4_string = \
@@ -784,30 +917,37 @@ class TestFromConfig(TestPycoast):
                     'borders': {'outline': (255, 0, 0), 'resolution': 'c'},
                     'rivers': {'outline': 'blue', 'resolution': 'c', 'level': 5}}
 
+        # Create the original cache file
         cache_filename = os.path.join(tmp, 'pycoast_cache_fakearea.png')
         img = cw.add_overlay_from_dict(overlays, area_def)
         res = np.array(img)
         self.assertTrue(fft_metric(euro_data, res),
                         'Writing of contours failed')
         self.assertTrue(os.path.isfile(cache_filename))
-
-        current_time = time.time()
-
-        img = cw.add_overlay_from_dict(overlays, area_def, current_time)
-
         mtime = os.path.getmtime(cache_filename)
 
+        # Reuse the generated cache file
+        img = cw.add_overlay_from_dict(overlays, area_def)
+        res = np.array(img)
+        self.assertTrue(fft_metric(euro_data, res),
+                        'Writing of contours failed')
+        self.assertTrue(os.path.isfile(cache_filename))
+        self.assertEqual(os.path.getmtime(cache_filename), mtime)
+
+        # Regenerate cache file
+        current_time = time.time()
+        cw.add_overlay_from_dict(overlays, area_def, current_time)
+        mtime = os.path.getmtime(cache_filename)
         self.assertGreater(mtime, current_time)
         self.assertTrue(fft_metric(euro_data, res),
                         'Writing of contours failed')
 
-        img = cw.add_overlay_from_dict(overlays, area_def, current_time)
-
+        cw.add_overlay_from_dict(overlays, area_def, current_time)
         self.assertEqual(os.path.getmtime(cache_filename), mtime)
         self.assertTrue(fft_metric(euro_data, res),
                         'Writing of contours failed')
         overlays['cache']['regenerate'] = True
-        img = cw.add_overlay_from_dict(overlays, area_def)
+        cw.add_overlay_from_dict(overlays, area_def)
 
         self.assertNotEqual(os.path.getmtime(cache_filename), mtime)
         self.assertTrue(fft_metric(euro_data, res),
